<?php

namespace Drupal\dynamic_library_loader\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for the Dynamic Library Loader module.
 */
class DynamicLibraryLoaderForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['dynamic_library_loader.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dynamic_library_loader_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    \Drupal::logger('dynamic_library_loader')->info('buildForm() called.');

    $config = $this->config('dynamic_library_loader.settings');

    // Initialize rows in the form state if not already set.
    if ($form_state->get('library_mappings') === NULL) {
      $form_state->set('library_mappings', $config->get('library_mappings') ?? []);
    }

    $entries = $form_state->get('library_mappings');

    // Wrap the table and button in a container for AJAX replacement.
    $form['library_mappings_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'dynamic-library-mappings-wrapper'],
    ];

    // Build the table structure.
    $form['library_mappings_wrapper']['library_mappings'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Context (Content Type/Taxonomy Vocabulary/Paragraph)'),
        $this->t('Theme/Module'),
        $this->t('Library'),
        $this->t('Actions'),
      ],
      '#empty' => $this->t('No mappings available. Add a new row.'),
    ];

    // Populate existing rows.
    foreach ($entries as $index => $entry) {
      $form['library_mappings_wrapper']['library_mappings'][$index]['context'] = [
        '#type' => 'textfield',
        '#default_value' => $entry['context'] ?? '',
        '#placeholder' => $this->t('e.g., article, image_gallery, tags'),
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['theme'] = [
        '#type' => 'textfield',
        '#default_value' => $entry['theme'] ?? '',
        '#placeholder' => $this->t('e.g., mytheme'),
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['library'] = [
        '#type' => 'textfield',
        '#default_value' => $entry['library'] ?? '',
        '#placeholder' => $this->t('e.g., mytheme/my-library'),
      ];
      $form['library_mappings_wrapper']['library_mappings'][$index]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => [[$this, 'removeRowSubmit']],
        '#name' => "remove_{$index}",
        '#attributes' => ['class' => ['button--small']],
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => [$this, 'ajaxCallback'],
          'wrapper' => 'dynamic-library-mappings-wrapper',
        ],
      ];
    }

    // Add a button to add new rows.
    $form['library_mappings_wrapper']['add_row'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Row'),
      '#submit' => [[$this, 'addRowSubmit']],
      '#ajax' => [
        'callback' => [$this, 'ajaxCallback'],
        'wrapper' => 'dynamic-library-mappings-wrapper',
      ],
    ];

    // Add the Save Configuration button outside of the AJAX container.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Configuration'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Submit handler for adding a row.
   */
  public function addRowSubmit(array &$form, FormStateInterface $form_state) {
    \Drupal::logger('dynamic_library_loader')->info('addRowSubmit() called.');
    $library_mappings = $form_state->get('library_mappings');
    $library_mappings[] = ['context' => '', 'theme' => '', 'library' => '']; // Add an empty row.
    $form_state->set('library_mappings', $library_mappings);
    $form_state->setRebuild();
  }

  /**
   * Submit handler for removing a row.
   */
  public function removeRowSubmit(array &$form, FormStateInterface $form_state) {
    \Drupal::logger('dynamic_library_loader')->info('removeRowSubmit() called.');
    $triggering_element = $form_state->getTriggeringElement();
    $name_parts = explode('_', $triggering_element['#name']);
    $index = end($name_parts);

    $library_mappings = $form_state->get('library_mappings');
    unset($library_mappings[$index]); // Remove the row.
    $form_state->set('library_mappings', $library_mappings);
    $form_state->setRebuild();
  }

  /**
   * AJAX callback to refresh the table and buttons.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    if (!isset($form['library_mappings_wrapper'])) {
      \Drupal::logger('dynamic_library_loader')->error('Missing library_mappings_wrapper in AJAX callback.');
      \Drupal::logger('dynamic_library_loader')->debug('<pre>' . print_r($form, TRUE) . '</pre>');
      return ['#markup' => $this->t('An error occurred. Please try again.')];
    }
    return $form['library_mappings_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $library_mappings = $form_state->getValue('library_mappings');
    foreach ($library_mappings as $index => $entry) {
      if (empty($entry['context']) || empty($entry['theme']) || empty($entry['library'])) {
        $form_state->setErrorByName("library_mappings][$index", $this->t('All fields in row %row must be filled out.', ['%row' => $index + 1]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    \Drupal::logger('dynamic_library_loader')->info('submitForm() called.');
    $library_mappings = $form_state->getValue('library_mappings');

    // Clean up empty rows and save valid mappings.
    $library_mappings = array_filter($library_mappings, function ($entry) {
      return !empty($entry['context']) && !empty($entry['theme']) && !empty($entry['library']);
    });

    // Save the mappings to configuration.
    $this->config('dynamic_library_loader.settings')
      ->set('library_mappings', $library_mappings)
      ->save();

    parent::submitForm($form, $form_state);
  }
}